import pygame

from effects import *
from graphics import *
from util import *
from away import *

class ExitState(Exception):
    pass

class NewState(Exception):
    def __init__(self,ns):
        self.state = ns

divx = 4
divy = 4
class State:
    def __init__(self):
        """Initialize a state"""
        self.world = []
        self.backdrop = None
        self.screen = pygame.display.get_surface()
        self.throw = None
        self.clock = pygame.time.Clock()
        self.dt = 0
    def sort_world(self):
        def cmp_sprite(sp1,sp2):
            return cmp(sp1.priority,sp2.priority)
        self.world.sort(cmp_sprite)
        for sprite in self.world:
            self.move(sprite)
    def move(self,sprite,pos=None):
        if pos:
            sprite.pos = pos[:]
    def _beforeloop(self):
        """What to do before we go into a loop"""
        self.updatedrects = []
    def loop(self):
        """Run until we should stop"""
        self._beforeloop()
        run = True
        ret = None
        while run:
            try:
                self.iteration()
            except ExitState:
                run = False
            except NewState,e:
                run = False
                ret = e.state
        img = self.screen.convert()
        fade(img,0.25,mode=-1,screen=self.screen)
        return ret
    def iteration(self):
        """One iteration"""
        if self.throw:
            raise self.throw
        self.dt = self.clock.tick(60)/1000.0
        self._beforerender()
        self._render()
        self._afterrender()
        self._events()
    def _beforerender(self):
        """Do stuff before we render"""
    def _render(self):
        """Update the display"""
        self.updatedrects = []
        changed = 0
        while changed < len(self.world):
            if self.world[changed].has_changed():
                break
            changed += 1
        clear = len(self.world)-1
        while clear >= changed:
            if self.world[clear].behind and \
                            self.world[clear].last_pos_drawn:
                self.screen.blit(self.world[clear].behind,
                        self.world[clear].last_pos_drawn)
                self.updatedrects.append([self.world[clear].last_pos_drawn,
                    self.world[clear].size])
            if self.world[clear].remove:
                self.world[clear].do_remove()
                self.world.remove(self.world[clear])
            clear -= 1
        draw = changed
        while draw < len(self.world):
            self.drawnew(self.world[draw])
            draw += 1
        #pygame.display.update(self.updatedrects)
        pygame.display.flip()
    def _events(self):
        """Low level event management"""
        for e in pygame.event.get():
            if e.type==pygame.QUIT:
                raise ExitState
            else:
                self.event(e)
    def _afterrender(self):
        """Do stuff when rendering is done"""
    def event(self,e):
        """Handle some event"""
    def assign_back(self,sprite,pos,size,surf=None):
        if not surf: surf = self.screen
        sprite.behind = pygame.Surface(sprite.size).convert()
        sprite.behind.blit(surf,[0,0],[sprite.pos,sprite.size])
    def drawnew(self,sprite,surf=None):
        """Clear old spot of sprite, update sprite position
        and draw sprite in new position"""
        if not surf:
            surf = self.screen
        self.updatedrects.append([sprite.pos,sprite.size])
        self.assign_back(sprite,sprite.pos,sprite.size,surf)
        sprite.last_pos_drawn = sprite.pos[:]
        sprite.dirty = False
        for img in sprite.images:
            img.paint(surf,sprite.pos)

class MouseState(State):
    """Any state that uses a mouse"""
    def __init__(self):
        State.__init__(self)
        self.cursor = Sprite()
        self.cursor.loadimage('gui/cursor.png')
        self.cursor.priority = 500
        self.cursor.over = []
        self.world.append(self.cursor)
    def event(self,e):
        State.event(self,e)
        if e.type==pygame.MOUSEMOTION:
            self.move(self.cursor,e.pos[:])
        if e.type==pygame.MOUSEBUTTONUP:
            for b in self.cursor.over:
                if hasattr(b,"mouse_click"):
                    b.mouse_click()
    def _beforerender(self):
        nothit = []
        hit = []
        for sprite in self.world:
            if sprite == self.cursor: continue
            if point_in_rect(self.cursor.pos,sprite.pos,sprite.size):
                hit.append(sprite)
            else:
                nothit.append(sprite)
        for sprite in hit:
            if sprite not in self.cursor.over:
                self.cursor.over.append(sprite)
                if hasattr(sprite,"mouse_enter"):
                    sprite.mouse_enter()
        for sprite in nothit:
            if sprite in self.cursor.over:
                self.cursor.over.remove(sprite)
                if hasattr(sprite,"mouse_exit"):
                    sprite.mouse_exit()
        State._beforerender(self)

class GameState(MouseState):
    def quit(self):
        self.throw = ExitState()
    def start(self):
        self.throw = NewState(InGameState)

from logic import *
import random
class InGameState(GameState):
    def __init__(self):
        GameState.__init__(self)
        pygame.mixer.music.load(data.filepath('music/song2.ogg'))
        pygame.mixer.music.play(0)
        
        self.schedule = timer_queue()
        self.schedule.add_recurring(self.update_hq,1,0.25)
        self.schedule.add_recurring(self.update_gate,1,1)
        self.schedule.add_recurring(self.makebandit,5,2)
        
        self.date = 0
        self.sec_per_unit = 5
        self.peoples = []
        
        self.rightwall = 1
        self.leftwall = -1

        self.academy = Academy(loc=-1)
        self.academy.sprite = BuildingSprite('acad')
        self.academy.sprite.setpos([350,0])
        self.hq = HQ(loc=0)
        self.hq.sprite = BuildingSprite('head')
        self.hq.sprite.setpos([35,0])
        self.storage = Storage(loc=1)
        self.storage.sprite = BuildingSprite('stor')
        self.storage.sprite.setpos([300,0])
        self.gate = Gate(loc=self.rightwall)

        self.oxygen = 100000

        self.taskmaster = TaskMaster()

        possible_people = ['bandit','carrier','civilian',
                'explorer','guard','medic','scientist']
        possible_people = ['civilian']
        for i in range(30):#range(35):
            people = Man(random.choice(possible_people))
            people.speed = 100
            people.click_event = self.killman(people)
            self.hq.peoples.append(people)
        
        self.ship = Ship(0)
        self.ship.sprite = ShipSprite('ship1')
        self.ship.add_metal(0)
        self.ship.sprite.pos = [100,0]
        self.shipbuilder = Builder(0,self.ship,self.taskmaster)

        self.builders = [self.shipbuilder]
        self.setloc(0)

        self.team = Team()

        #oldscr = self.screen
        #newscr = self.screen = pygame.Surface([640,480])
        #self._render()
        #fade(newscr,3)
        #self.screen = oldscr
        #self.screen.blit(newscr,[0,0])
    def iteration(self):
        #If this is our first run, reset the clock
        #so that the fade time doesn't count in dt
        rc = getattr(self,'_resetclock',0)
        if not rc:
            oldscr = self.screen
            self.screen = pygame.Surface([640,480])
        if rc<2:
            self.clock = pygame.time.Clock()
        if rc==1:
            self._resetclock = 2
        GameState.iteration(self)
        if not rc:
            fade(self.screen,1,screen=oldscr)
            self.screen = oldscr
            self._resetclock = 1
    def killman(self,man=None):
        def f(man=man):
            if not man.alive: return
            man.set_frame(0,'dead',False)
            man.alive = False
            playsound(man.dead_sound,True,1)
            #Put bodies to back layer so they don't have to be updated
            man.priority = -100
            if man in self.world:
                self.world.remove(man)
                self.world.insert(self.peoples_entrance,man)
                self.peoples_entrance += 1   #Make new people come out in front
            #of the dead body
        return f
    def update_hq(self):
        """Should be run every second, to spit out one
        people per second"""
        if self.hq.peoples:
            people = self.hq.peoples.pop(-1)
            people.dir = -1
            people.pos = [100,480-8-30]
            self.peoples.append(people)
            if self.loc == people.loc:
                self.world.insert(self.peoples_entrance,people)
    def update_gate(self):
        """Should be run every second, to spit out one
        people per second"""
        if self.gate.peoples:
            people = self.gate.peoples.pop(-1)
            people.last_carry = None
            print "from gate carry:",people.carry
            people.dirty = True
            people.set_frame(0,"main")
            people.dir = -1
            people.pos = [640,480-8-30]
            self.peoples.append(people)
            if self.loc == people.loc:
                self.world.insert(self.peoples_entrance,people)
            print people.carry
    def makebandit(self):
        guards = 0
        notbandits = []
        for people in self.peoples:
            if people.job is not "bandit" and people.alive:
                notbandits.append(people)
            if people.job == "guard" and people.alive:
                guards +=1
        if not notbandits: return
        percent = (guards+10)/float(len(self.peoples))
        if random.random() > percent:
            makethisguybandit = random.choice(notbandits)
            makethisguybandit.setjob("bandit")
    def infront(self,people):
        """Returns someone in front of the people"""
        for target in self.peoples:
            if not target.alive:
                continue
            if target == people:
                continue
            if target.loc != people.loc:
                continue
            if people.dir>0:
                if target.pos[0]>people.pos[0]:
                    if target.pos[0]<60+people.pos[0]:
                        return target
            if people.dir<0:
                if target.pos[0]<people.pos[0]:
                    if target.pos[0]>people.pos[0]-60:
                        return target
    def _beforerender(self):
        GameState._beforerender(self)
        self.date += self.dt
        self.schedule.process(self.dt)
        for building in [self.hq]:
            self.oxygen -= len(building.peoples)*self.dt
        currentlocmen = [x for x in self.world if isinstance(x,Man)]
        for man in self.peoples:
            man.add_frame(self.dt)
            if man.alive:
                self.oxygen -= self.dt
                if self.date - getattr(man,'last_march',0) > 0.5:
                    playsound(man.walk_sound,False,
                            random.randint(1,3)/10.0)
                    man.last_march = self.date
                np = [man.pos[0]+self.dt*man.speed*man.dir,man.pos[1]]
                if np[0]>640:
                    if man.loc == self.rightwall:
                        man.dir = -1
                    else:
                        man.loc += 1
                        np[0] = 0
                        if man in self.world:
                            man.set_remove()
                        if man.loc == self.loc and man not in self.world:
                            self.world.insert(self.peoples_entrance,man)
                    #if man.job == 'carrier':
                        #if man.carry == None:
                            #man.carry = random.choice(['metal','metal','fuel'])
                if np[0]<-30:
                    if man.loc == self.leftwall:
                        man.dir = 1
                    else:
                        man.loc -= 1
                        np[0] = 640
                        if man in self.world:
                            man.set_remove()
                        if man.loc == self.loc and man not in self.world:
                            self.world.insert(self.peoples_entrance,man)
                if np[0]>self.ship.sprite.pos[0] and np[0]<self.ship.sprite.pos[0]+100 \
                    and man.carry and man.loc==self.ship.loc:
                    if man.carry == 'metal':
                        print 'metal to ship'
                        self.shipbuilder.metal_store += 1
                        man.carry = None
                    elif man.carry == 'fuel':
                        self.shipbuilder.fuel_store += 1
                        man.carry = None
                if np[0]>self.academy.sprite.pos[0] and np[0]<self.academy.sprite.pos[0]+100:
                    if man.loc == self.academy.loc:
                        if man.job == 'civilian' and self.academy.queue:
                            job = self.academy.queue.pop(0)
                            man.setjob(job)
                if np[0]>self.storage.sprite.pos[0] and np[0]<self.storage.sprite.pos[0]+100:
                    if man.loc == self.storage.loc:
                        if man.job == "carrier":
                            if not man.carry:
                                r = self.storage.getresource()
                                if r:
                                    man.carry = r
                if man.job == "bandit":
                    target = self.infront(man)
                    if target and target.job!='bandit':
                        self.killman(target)()
                if man.job == "guard":
                    target = self.infront(man)
                    if target and target.job=='bandit':
                        self.killman(target)()
                self.move(man,np)
        if self.oxygen <= 0:
            self.oxygen = 0
            if random.randint(0,1000)>950:
                alivemen = [x for x in self.peoples \
                    if x.alive]
                if alivemen:
                    people = random.choice(alivemen)
                    people.alive = False
                    people.set_frame(0,'choke',False)
                    if people.loc == self.loc:
                        playsound(people.choke_sound,True,1)
        self.oxygen_g.updatetext('Oxygen=%d'%self.oxygen)
        self.oxygen_g.loadimage('gui/wide_line.png')
        self.date_g.updatetext('Date:%d'%(self.sec_per_unit*self.date))
        self.date_g.loadimage('gui/wide_line.png')
        if self.car:
            self.car.updatetext('train carriers %s'%\
                    len([x for x in self.academy.queue if x=='carrier']))
            self.car.loadimage('gui/button_1.png')
            self.grd.updatetext('train guards %s'%\
                    len([x for x in self.academy.queue if x=='guard']))
            self.grd.loadimage('gui/button_1.png')
            self.acex.updatetext('train explor %s'%\
                    len([x for x in self.academy.queue if x=='explorer']))
            self.acex.loadimage('gui/button_1.png')
            self.acme.updatetext('train medics %s'%\
                    len([x for x in self.academy.queue if x=='medic']))
            self.acme.loadimage('gui/button_1.png')
        if getattr(self,"awcar",None):
            self.awcar.updatetext('send carriers(%s)'%\
                    len([x for x in self.team.waiting if\
                        x.job=='carrier']))
            self.awcar.loadimage('gui/button_1.png')
        if getattr(self,"awgu",None):
            self.awgu.updatetext('send guards(%s)'%\
                    len([x for x in self.team.waiting if\
                        x.job=='guard']))
            self.awgu.loadimage('gui/button_1.png')
        if getattr(self,"awmed",None):
            self.awmed.updatetext('send medics(%s)'%\
                    len([x for x in self.team.waiting if\
                        x.job=='medic']))
            self.awmed.loadimage('gui/button_1.png')
        if getattr(self,"awex",None):
            self.awex.updatetext('send explorer(%s)'%\
                    len([x for x in self.team.waiting if\
                        x.job=='explorer']))
            self.awex.loadimage('gui/button_1.png')
        for builder in self.builders:
            builder.update(self.dt)
    def setloc(self,loc):
        self.cursor.over = []
        self.loc = loc
        self.screen.fill([0,0,0])
        #pygame.display.flip()
        for sprite in self.world:
            sprite.do_remove()
        self.world = []
        self.peoples_entrance = 0
        #add background
        self.sky = Sprite()
        self.sky.loadimage('backdrops/sky2.png')
        self.world.append(self.sky)
        self.peoples_entrance += 1
        dome = Sprite()
        dome.loadimage('backdrops/dome.png')
        self.world.append(dome)
        self.peoples_entrance += 1

        #add ship, buildings
        if loc==0:
            self.world.append(self.ship.sprite)
            self.world.append(self.hq.sprite)
            self.peoples_entrance += 2
        if loc==-1:
            self.world.append(self.academy.sprite)
        if loc==1:
            self.world.append(self.storage.sprite)
        #remember spot
        self.peoples_entrance = len(self.world)
        for people in self.peoples:
            if self.loc == people.loc:
                self.world.insert(self.peoples_entrance,people)
        #go through people and add ones on this screen
        
        #add foreground
        #add menu
        ad = Button()
        ad.pos = [0,40]
        ad.updatetext("Academy")
        ad.loadimage("gui/button_1.png")
        self.world.append(ad)
        ad.click_event = self.set_locm1
        hq = Button()
        hq.pos = [200,40]
        hq.updatetext("Headquarters")
        hq.loadimage("gui/button_1.png")
        self.world.append(hq)
        hq.click_event = self.set_loc0
        ss = Button()
        ss.pos = [400, 40]
        ss.updatetext("Storage")
        ss.loadimage("gui/button_1.png")
        self.world.append(ss)
        ss.click_event = self.set_loc1
        #location specific buttons
        self.car=None
        if loc == -1:
            car = Button()
            car.pos = [0,80]
            car.updatetext("Carriers")
            car.loadimage("gui/button_1.png")
            self.world.append(car)
            car.click_event = self.academy.add_carrier
            self.car=car
            grd = Button()
            grd.pos = [0,120]
            grd.updatetext("Guards")
            grd.loadimage("gui/button_1.png")
            self.world.append(grd)
            grd.click_event = self.academy.add_guard
            self.grd=grd
            self.acex = Button([0,160],"","gui/button_1.png",self.world)
            self.acex.click_event = self.academy.add_explorer
            self.acme = Button([0,200],"","gui/button_1.png",self.world)
            self.acme.click_event = self.academy.add_medic
        if loc == 0:
            self.away = Button([400,120],"away","gui/button_1.png",self.world)
            self.away.click_event = self.goaway
            self.clr = Button([200,120],"Clear team","gui/button_1.png",
                    self.world)
            self.clr.click_event = self.clearteam
            self.awcar = Button([0,80],"send carrier","gui/button_1.png",
                    self.world)
            self.awcar.click_event = self.team_add_carrier
            self.awgu = Button([200,80],"send guard","gui/button_1.png",
                    self.world)
            self.awgu.click_event = self.team_add_guard
            self.awmed = Button([400,80],"send medic","gui/button_1.png",
                    self.world)
            self.awmed.click_event = self.team_add_medic
            self.awex = Button([0,120],"send explorer","gui/button_1.png",
                    self.world)
            self.awex.click_event = self.team_add_explorer
        #add mouse again
        self.world.append(self.cursor)
        #Add interface
        self.oxygen_g = Text()
        self.oxygen_g.updatetext('O2 = ')
        self.oxygen_g.loadimage('gui/wide_line.png')
        self.world.append(self.oxygen_g)
        self.date_g = Text()
        self.date_g.pos = [320,0]
        self.date_g.updatetext('date:')
        self.date_g.loadimage('gui/wide_line.png')
        self.world.append(self.date_g)


    def set_loc1(self):
        self.setloc(1)
    
    def set_loc0(self):
        self.setloc(0)
    
    def set_locm1(self):
        self.setloc(-1)
        
    def clearteam(self):
        self.team.teammembers = []
        for p in self.team.waiting:
            #Make sure people can have a task again
            pass
        self.team.waiting = []

    def team_add_carrier(self):
        possible = [x for x in self.peoples if x.job=='carrier']
        for p in possible:
            if p.carry: continue
            if p in self.team.waiting: continue
            self.team.waiting.append(p)
            return
    def team_add_guard(self):
        possible = [x for x in self.peoples if x.job=='guard']
        for p in possible:
            if p.carry: continue
            if p in self.team.waiting: continue
            self.team.waiting.append(p)
            return
    def team_add_medic(self):
        possible = [x for x in self.peoples if x.job=='medic']
        for p in possible:
            if p.carry: continue
            if p in self.team.waiting: continue
            self.team.waiting.append(p)
            return
    def team_add_explorer(self):
        possible = [x for x in self.peoples if x.job=='explorer']
        for p in possible:
            if p.carry: continue
            if p in self.team.waiting: continue
            self.team.waiting.append(p)
            return


    def goaway(self):
        team = self.team
        for people in team.waiting:
            if people in self.peoples:
                team.teammembers.append(people.job)
                self.peoples.remove(people)
                team.y = people.pos[1]
                people.set_remove()
            else:
                print "error?"
        runmission(team,1,random.randint(1,3))
        self.oxygen -= team.triptime*5
        self.schedule.insert(self.comeback(team),next=team.triptime)
        self.team = Team()
            
        
    def comeback(self,team):
        def f():
            for job in team.teammembers:
                people = Man(job)
                people.loc = self.rightwall
                people.pos[0] = 640
                people.pos[1] = team.y
                if job == "carrier" and team.resources:
                    people.carry = team.resources[0]
                    print people.carry
                    del team.resources[0]
                self.gate.peoples.append(people)
        return f
                    
